#NoTrayIcon
#pragma compile(Icon, icons\Speed Browse.ico)
#pragma compile(UPX, False)
#pragma compile(FileDescription, Speed Browse)
#pragma compile(ProductName, Speed Browse)
#pragma compile(FileVersion, 1.2.1.13)
#pragma compile(LegalCopyright, 2014 David Spivey)
#pragma compile(CompanyName, 'Spy-V Software Developement')
#pragma compile(AutoItExecuteAllowed, true)

#cs
	The function of this app is to dynamically load Speed Browse code as 32 or 64-bit as necessary. In order to do this, we compile the main Speed Browse
	app as a3x, renamed to a dll. A3x is a compressed au3 file that can be run in the context of its calling app. This app is 32-bit, and if the OS it is
	running inside of is 32-bit, we fileinstall and run the a3x "dll" by running ourself with the commandline /AutoIt3ExecuteScript "our dll". If the OS
	is 64-bit, we fileinstall the 64-bit runner with the same command line. This app stays around in the background in both circumstances to enable
	automated updates.
#ce

;	Commonly-used variables among Speed Browse, Speed Browse x64 and Speed Browse "dll"
#include "SBCommon.au3"
#include "..\ProjectIncludes\IPCLib.au3" ; Interprocess communication library for sending messages between running instances.

#include <FileConstants.au3>
#include <StringConstants.au3>
#include <Misc.au3> ; Singleton
_Singleton('$appName$ Launcher') ; assure only one instance runs.
Global Enum $fav0, $favPath, $favIcon, $favOrganizeFolder ; $fav0 is a placeholder for the 0 item of a StringSplit. The remaining are used in MenuAddFavorites

UpgradeINI()
Func UpgradeINI()
	; Migrate preferences to new ini, and favorites to new folders, so they will be available for Speed Browse "dll"
	Local $oldINIfile = '@LocalAppDataDir@\' & StringStripWS($appName, $STR_STRIPALL) & 'Prefs.ini'
	If FileExists($oldINIfile) Then
		Local $oldINI = _IniLoad($oldINIfile);								Read the old INI
		Local $oldPrefs = _IniReadSection($oldINI, "Preferences");			Grab the preferences from old INI
		_IniWriteSection($iniPrefs, "Preferences", $oldPrefs);				Write to the new INI
		Local $oldFavs = _IniReadSection($oldINI, "Favorites");				Check for the existence of favorites in the old INI
		If Not @error Then;													If we have an error, the section does not exist.
			Local $newArr = $oldFavs;										Make a copy of the _IniReadSection favorites variable so we can directly reassign values
			For $i = 1 To $newArr[0][0]
				$newArr[$i][1] = StringSplit($newArr[$i][1], "|");			favorites are "folder path|iconfile,number|organization folder"
			Next

			DirCreate($favDir);												Build folder structure
			Local $fav, $fol, $mnu, $submenu
			For $i = 1 To $newArr[0][0]
				$fav = $newArr[$i][1]
				If $fav[0] <> 3 Then ContinueLoop
				$fol = $fav[$favOrganizeFolder]
				If $fol <> "" Then
					DirCreate("$favDir$\$fol$")
				EndIf
			Next

			Local $name, $fav, $iconSplit, $icon, $fol, $mnu
			For $i = 1 To $newArr[0][0]
				$name = $newArr[$i][0]
				$fav = $newArr[$i][1]
				If $fav[0] <> 3 Then ContinueLoop
				$fol = ($fav[$favOrganizeFolder] = '') ? '' : '\' & $fav[$favOrganizeFolder]
				$iconSplit = StringSplit($fav[$favIcon], ",")
				If $iconSplit[0] <> 2 Then ContinueLoop
				FileCreateShortcut($fav[$favPath], "$favDir$$fol$\$name$.lnk", "", "", "", $iconSplit[1], "", $iconSplit[2])
			Next
		EndIf
		FileDelete($oldINIfile)
	EndIf
EndFunc   ;==>UpgradeINI

UpgradeFavorites()
Func UpgradeFavorites()
	; If favorites have already been converted from ini to folders, migrate favorites to new folder, so they will be available for Speed Browse "dll"
	Local $oldfavDir = '@LocalAppDataDir@\' & StringStripWS($appName, $STR_STRIPALL) & 'Favorites'
	If FileExists($oldfavDir) Then
		DirRemove($favDir, 1) ; we have to remove the existing folder in order to rename this one as we move it.
		DirMove($oldfavDir, $favDir, $FC_OVERWRITE)
	EndIf
EndFunc   ;==>UpgradeFavorites

_IniWrite($iniPrefs, "Preferences", "Launch", @ScriptFullPath) ; Tell child apps which app to launch for startup menus and shortcuts.
If _IniRead($iniPrefs, "Preferences", "Version", "0.0.0.0") <> $SBVersion Then ; We need to upgrade versions
	_IPC_Start("Server", $appName) ; Use IPC broadcast to close existing software. This allows it to be closed safely without losing data.
	_IPC_Broadcast("Exit")
	For $i = 0 To 28 ; Wait 7 seconds (or less) for processes to close gracefully.
		Sleep(250)
		If ProcessList("Speed Browse.exe")[0][0] <= 1 And Not ProcessExists('Speed Browse x64.exe') Then ExitLoop ; If we are the only Speed Browse hanging around then go on.
	Next
	FileInstall("Speed Browse.dll", $appDll, $FC_OVERWRITE) ; Replace / Install DLL
	FileInstall("Speed Browse x64.exe", '$appDir$\Speed Browse x64.exe', $FC_OVERWRITE) ; Replace / Install x64 exe
	_IniWrite($iniPrefs, "Preferences", "Version", $SBVersion) ; Update version in Preferences.
EndIf
If @OSArch <> "X86" Then ; We use OS architecture instead of CPU architecture because we may have a 32-bit version of Windows running on a 64-bit CPU.
	Run('"$appDir$\Speed Browse x64.exe" /AutoIt3ExecuteScript "$appDll$" ' & StringReplace($CmdLineRaw, @ScriptFullPath, ''))
Else
	Run('"@ScriptFullPath@" /AutoIt3ExecuteScript "$appDll$" ' & StringReplace($CmdLineRaw, @ScriptFullPath, ''))
EndIf
; Wait until other Speed Browse executables are closed.
; We do this because we are the updater, and automatically get restarted when new versions are created. However, when the user-facing interface
; is closed, we do the "right thing".

Local $count = 0
While 1
	Sleep(6000)
	If ProcessList("Speed Browse.exe")[0][0] <= 1 And Not ProcessExists('Speed Browse x64.exe') Then
		$count += 1
	Else
		$count = 0
	EndIf
	If $count = 2 Then Exit
WEnd

Do
	Sleep(6000)
Until ProcessList("Speed Browse.exe")[0][0] <= 1 And Not ProcessExists('Speed Browse x64.exe')
